@@ -75,6 +75,7 @@ typedef struct _VteAccessibleText
         VteTerminal               *terminal;
         VteAccessibleTextContents  contents[2];
         guint                      contents_flip : 1;
+        guint                      text_scrolled : 1;
 } VteAccessibleText;
 
 static inline gboolean
@@ -318,9 +319,9 @@ vte_accessible_text_contents_snapshot (VteAccessibleTextContents *contents,
                  * it's a new line and we need to keep track of where
                  * it is. */
                 if ((i == 0) || (attrs->row != row)) {
-                        _vte_debug_print (VTE_DEBUG_ALLY,
-                                          "Row %d/%ld begins at %d.\n",
-                                          int(char_positions_get_size (&contents->linebreaks)),
+                        _vte_debug_print (vte::debug::category::ALLY,
+                                          "Row {}/{} begins at {}",
+                                          int(char_positions_get_size(&contents->linebreaks)),
                                           attrs->row, i);
                         char_positions_append (&contents->linebreaks, &i);
                 }
@@ -334,7 +335,9 @@ vte_accessible_text_contents_snapshot (VteAccessibleTextContents *contents,
         /* Update the caret position. */
         long ccol, crow;
         vte_terminal_get_cursor_position (terminal, &ccol, &crow);
-        _vte_debug_print (VTE_DEBUG_ALLY, "Cursor at (%ld, " "%ld).\n", ccol, crow);
+        _vte_debug_print (vte::debug::category::ALLY,
+                          "Cursor at ({}, {})",
+                          ccol, crow);
         gsize caret = vte_accessible_text_contents_find_caret (contents, ccol, crow);
 
         contents->n_bytes = gstr->len;
@@ -344,9 +347,9 @@ vte_accessible_text_contents_snapshot (VteAccessibleTextContents *contents,
         contents->cached_caret_column = ccol;
         contents->cached_caret_row = crow;
 
-        _vte_debug_print (VTE_DEBUG_ALLY,
+        _vte_debug_print (vte::debug::category::ALLY,
                           "Refreshed accessibility snapshot, "
-                          "%ld cells, %ld characters.\n",
+                          "{} cells, {} characters",
                           long(vte_char_attr_list_get_size(&contents->attrs)),
                           long(char_positions_get_size (&contents->characters)));
 }
@@ -473,14 +476,14 @@ vte_accessible_text_get_contents_at (GtkAccessibleText            *accessible,
                      line++) {
                         guint line_offset = *char_positions_index (&contents->linebreaks, line);
 
-                        if (line_offset > char_offset) {
+                        if (line_offset > offset) {
                                 line--;
                                 break;
                         }
                 }
 
-                _vte_debug_print (VTE_DEBUG_ALLY,
-                                  "Character %u is on line %u.\n",
+                _vte_debug_print (vte::debug::category::ALLY,
+                                  "Character {} is on line {}",
                                   offset, line);
 
                 *start = *char_positions_index (&contents->linebreaks, line);
@@ -685,12 +688,14 @@ vte_accessible_text_get_attributes (GtkAccessibleText        *accessible,
 
         *attribute_names = g_new0 (char *, n_attrs + 1);
         *attribute_values = g_new0 (char *, n_attrs + 1);
-        *n_ranges = 1;
-        *ranges = (GtkAccessibleTextRange *)g_memdup2 (&range, sizeof range);
+        *n_ranges = n_attrs;
+        *ranges = g_new (GtkAccessibleTextRange, n_attrs);
 
         for (i = 0; i < n_attrs; i++) {
                 (*attribute_names)[i] = g_strdup (attrs[i].name);
                 (*attribute_values)[i] = g_strdup (attrs[i].value);
+                (*ranges)[i].start = range.start;
+                (*ranges)[i].length = range.length;
         }
 
         return TRUE;
@@ -814,6 +819,11 @@ vte_accessible_text_contents_changed (VteTerminal       *terminal,
         if (!vte_terminal_get_enable_a11y (terminal))
           return;
 
+        if (state->text_scrolled) {
+                state->text_scrolled = FALSE;
+                return;
+        }
+
         prev = &state->contents[state->contents_flip];
         next = &state->contents[!state->contents_flip];
 
@@ -837,12 +847,11 @@ vte_accessible_text_contents_changed (VteTerminal       *terminal,
         /* NOTE:
          *
          * The code below is based upon what vteaccess.cc did for GTK 3.
-         * It does not do any sort of appropriate diffing to try to handle
-         * scrolling correctly. That would be a good idea to implement in
-         * the longer term.
          *
          * It just looks for a long prefix match, and then a long suffix
          * match and attempts to diff what is between those to end points.
+         *
+         * Scrolling based changes are handled separately.
          */
 
         const char *prevc = prevstr;
@@ -926,7 +935,9 @@ vte_accessible_text_cursor_moved (VteTerminal       *terminal,
                 return;
         }
 
-        _vte_debug_print (VTE_DEBUG_ALLY, "Cursor at (%ld, " "%ld).\n", ccol, crow);
+        _vte_debug_print (vte::debug::category::ALLY,
+                          "Cursor at ({}, {})",
+                          ccol, crow);
 
         contents->cached_caret_column = ccol;
         contents->cached_caret_row = crow;
@@ -1013,3 +1024,81 @@ _vte_accessible_text_init (GtkAccessibleText *accessible)
                                         GTK_ACCESSIBLE_PROPERTY_MULTI_LINE, TRUE,
                                         GTK_ACCESSIBLE_VALUE_UNDEFINED);
 }
+
+void
+_vte_accessible_text_scrolled (GtkAccessibleText *accessible, long delta)
+{
+        VteTerminal *terminal = VTE_TERMINAL (accessible);
+        VteAccessibleText *state = vte_accessible_text_get (terminal);
+        VteAccessibleTextContents *prev = &state->contents[state->contents_flip];
+        VteAccessibleTextContents *next = &state->contents[!state->contents_flip];
+
+        g_assert (VTE_IS_TERMINAL (terminal));
+        g_assert (state != nullptr);
+        g_assert (state->terminal == terminal);
+
+        if (!vte_terminal_get_enable_a11y (terminal))
+                return;
+
+        _vte_debug_print (vte::debug::category::ALLY,
+                          "Text scrolled by {} lines", delta);
+
+        vte_accessible_text_contents_reset (next);
+        vte_accessible_text_contents_snapshot (next, state->terminal);
+
+        if (delta > 0) {
+                /* Scrolling down: lines at the top disappeared, new lines appeared at bottom */
+                gsize lines_to_remove = MIN(delta, char_positions_get_size(&prev->linebreaks));
+                if (lines_to_remove > 0 && char_positions_get_size(&prev->linebreaks) > 0) {
+                        /* Find how many characters were in the removed lines */
+                        gsize chars_removed = 0;
+                        if (lines_to_remove < char_positions_get_size(&prev->linebreaks)) {
+                                chars_removed = *char_positions_index(&prev->linebreaks, lines_to_remove);
+                        } else {
+                                chars_removed = prev->n_chars;
+                        }
+
+                        if (chars_removed > 0) {
+                                /* Notify that text was removed from the beginning */
+                                gtk_accessible_text_update_contents(accessible,
+                                                                    GTK_ACCESSIBLE_TEXT_CONTENT_CHANGE_REMOVE,
+                                                                    0, chars_removed);
+                                
+                                /* Notify that new text was added at the end */
+                                state->contents_flip = !state->contents_flip;
+                                gtk_accessible_text_update_contents(accessible,
+                                                                    GTK_ACCESSIBLE_TEXT_CONTENT_CHANGE_INSERT,
+                                                                    MAX(0L, (long)next->n_chars - (long)chars_removed),
+                                                                    next->n_chars);
+                        }
+                }
+        } else if (delta < 0) {
+                /* Scrolling up: lines at the bottom disappeared, new lines appeared at top */
+                gsize lines_to_remove = MIN(-delta, char_positions_get_size(&prev->linebreaks));
+                if (lines_to_remove > 0 && char_positions_get_size(&prev->linebreaks) > 0) {
+                        /* Find how many characters were in the removed lines from bottom */
+                        gsize start_remove = char_positions_get_size(&prev->linebreaks) - lines_to_remove;
+                        gsize chars_removed = 0;
+                        if (start_remove < char_positions_get_size(&prev->linebreaks)) {
+                                gsize remove_start_pos = *char_positions_index(&prev->linebreaks, start_remove);
+                                chars_removed = prev->n_chars - remove_start_pos;
+                        } else {
+                                chars_removed = prev->n_chars;
+                        }
+
+                        if (chars_removed > 0) {
+                                /* Notify that text was removed from the end */
+                                gtk_accessible_text_update_contents(accessible,
+                                                                    GTK_ACCESSIBLE_TEXT_CONTENT_CHANGE_REMOVE,
+                                                                    prev->n_chars - chars_removed, prev->n_chars);
+                                
+                                /* Notify that new text was added at the beginning */
+                                state->contents_flip = !state->contents_flip;
+                                gtk_accessible_text_update_contents(accessible,
+                                                                    GTK_ACCESSIBLE_TEXT_CONTENT_CHANGE_INSERT,
+                                                                    0, chars_removed);
+                        }
+                }
+        }
+        state->text_scrolled = TRUE;
+}
